<?php
App::uses('BuzzWaiversAppController', 'BuzzWaivers.Controller');
App::uses('CakeEvent', 'Event');

class WaiversController extends BuzzWaiversAppController {

	public $kiosk = false;
	public $participant;
	public $booking;

	public $activeStage;

	public function __construct(CakeRequest $Request, CakeResponse $Response) {
		$this->components['PreStage'] = array(
			'className' => 'BuzzWaivers.PreStage'
		);
		$this->components['PostStage'] = array(
			'className' => 'BuzzWaivers.PostStage'
		);

		parent::__construct($Request, $Response);
	}

	public function beforeFilter() {
		parent::beforeFilter();

		$this->Auth->allow(array(
			'complete_booking',
			'manage',
			'participants',
			'stage'
		));

		// default stage process
		$stageProcess = Configure::read('BuzzWaivers.stageProcess');

		if ($this->kiosk) {
			$stageProcess = 'kiosk';
		}

		$stages = Configure::read('BuzzWaivers.stages');
		// check for an overriding process
		if ($this->Session->check('BuzzWaivers.stageProcess')) {
			$stageProcess = $this->Session->read('BuzzWaivers.stageProcess');
		}

		// check the selected process to use is valid
		if (empty($stages[$stageProcess]) || ! is_array($stages[$stageProcess])) {
			throw new InvalidStageProcessException(array($stageProcess));
		}
		$this->activeStageProcess = $stageProcess;
	}

	/**
	 * loop the given array and try to run the methods on the given
	 * component - for processing the stages
	 *
	 * @param 	string  $componentKey 	The component key to run methods on
	 * @param 	array 	$processors 		Array of any methods to run
	 * @return 	bool
	 */
	protected function _callStageProcessors($componentKey, $processors) {
		$result = true;
		if (! empty($processors)) {
			foreach ($processors as $method) {
				if (method_exists($this->{$componentKey}, $method)) {
					$result = $result && $this->{$componentKey}->{$method}();
				}
			}
		}
		return $result;
	}

	/**
	 * preStage failed, calculate the redirect route
	 *
	 * @param 	string 	$stageId 	The current stage ID
	 * @return 	array 	$route 		The route array to redirect to
	 */
	public function calculatePreStageRedirect($stageId) {
		$stages = Configure::read('BuzzWaivers.stages.' . $this->activeStageProcess);
		// all processors returned true, proceed
		$position = array_search($stageId, array_keys($stages));
		$position--;
		if ($position >= 0) {
			$stageIterator = new ArrayIterator($stages);
			$stageIterator->seek($position);
			$prevStageKey = $stageIterator->key();
			// redirect to the next stage
			if (! empty($prevStageKey)) {

				$route = array(
					'admin' => false,
					'plugin' => 'buzz_waivers',
					'controller' => 'waivers',
					'action' => 'stage',
				);

				$route[] = $prevStageKey;
			}
		}
		if (empty($route)) {
			// no stage found - start from beginning

			$route =  array(
				'admin' => false,
				'plugin' => 'buzz_waivers',
				'controller' => 'waivers',
				'action' => 'participants'
			);
		}

		return $route;
	}
	/**
	 * postStage succeeded, calculate the redirect route
	 *
	 * @param 	string 	$stageId 	The current stage ID
	 * @return 	array 	$route 	The route array to redirect to
	 */
	public function calculatePostStageRedirect($stageId) {
		$stages = Configure::read('BuzzWaivers.stages.' . $this->activeStageProcess);
		// all processors returned true, proceed
		$position = array_search($stageId, array_keys($stages));
		$stageIterator = new ArrayIterator($stages);
		$stageIterator->seek($position);
		$stageIterator->next();
		$nextStageKey = $stageIterator->key();
		// redirect to the next stage
		if (! empty($nextStageKey)) {
			$route = array(
				'admin' => false,
				'plugin' => 'buzz_waivers',
				'controller' => 'waivers',
				'action' => 'stage',
			);
			$route[] = $nextStageKey;
		} else {
			// no more stages but everything was successful.
			// complete
			$route =  array(
				'admin' => false,
				'plugin' => 'buzz_waivers',
				'controller' => 'waivers',
				'action' => 'participants'
			);
		}

		return $route;
	}

	/**
	 * the main stage method to load each stage of the waiver
	 *
	 */
	public function stage($stageId, $id) {
		$stageInfo = Configure::read('BuzzWaivers.stages.' . $this->activeStageProcess . '.' . $stageId);

		$stages = Configure::read('BuzzWaivers.stages.' . $this->activeStageProcess);

		$stageCount = count($stages)-1; // We dont count the last stage
		$currentStageCount = 1;
		$i=1;
		foreach($stages as $key => $stage) {
			if ($key == $stageId) {
				$currentStageCount = $i;
			}
			$i++;
		}

		$this->set(compact('currentStageCount', 'stageCount'));

		if (empty($stageInfo)) {

			$route = array(
				'action' => 'index'
			);

			if ($this->kiosk) {
				$route['kiosk'] = 'kiosk';
			}

			$this->redirect($route);
		}
		// set the active stage
		$this->activeStage = $stageId;
		// check the preStage calls
		$result = $this->_callStageProcessors('PreStage', $stageInfo['pre']);
		// go back a step if the results are false
		if ($result === false) {
			$route = $this->calculatePreStageRedirect($stageId);
			$route[] = $id;

			$route = Router::url($route);

			if ($this->kiosk) {
				$route = '/kiosk' . $route;
			}

			$this->redirect(
				$route
			);
		}

		if ($this->request->is('post') || $this->request->is('put')) {
			$result = $this->_callStageProcessors('PostStage', $stageInfo['post']);
			// proceed to next step
			if ($result === true) {
				$route = $this->calculatePostStageRedirect($stageId);
				$route[] = $id;

				$route = Router::url($route);

				if ($this->kiosk) {
					$route = '/kiosk' . $route;
				}

				$this->redirect(
					$route
				);
			}
		}

		$this->view = 'BuzzWaivers.' . $stageInfo['template'];

		$route = array(
			'plugin' => 'buzz_waivers',
			'controller' => 'waivers',
			'action' => 'participants',
			$id
		);

		if ($this->kiosk) {
			$route['kiosk'] = 'kiosk';
		}

		$cancelUrl = Router::url($route);

		$this->set(compact('cancelUrl'));
	}

	public function complete_booking($salesRef, $date) {
		if ($this->request->is('post')) {

			$date = date('Y-m-d H:i:s', $date);
			$this->loadModel('BuzzWaivers.WaiversApi');
			$this->loadModel('BuzzWaivers.Participant');

			// Check the booking exists
			$data = $this->WaiversApi->getBooking($salesRef, $date);

			if (empty($data)) {
				$this->Session->setFlash(
					__d('buzz_waivers', 'There was a problem validating the participant details.'),
					'flash_fail'
				);

				return $this->redirect($this->referer());
			}

			// Check for existing waiver participant records
			$participants = $this->Participant->find('all', array(
				'conditions' => array(
					'sales_ref' => $salesRef
				)
			));

			$storedParticipants = (! empty($participants) ? count($participants) : 0);
			$maxParticipants = (! empty($data[0]['participants']) ? $data[0]['participants'] : 0);

			if ($storedParticipants >= $maxParticipants) {

				$route = array(
					'plugin' => 'buzz_check_bookings',
					'controller' => 'check_bookings',
					'action' => 'check'
				);

				if ($this->kiosk) {
					$route['kiosk'] = 'kiosk';
				}

				$checkBookingUrl = Router::url($route);

				$this->Session->setFlash(
					__d('buzz_waivers', 'All participant slots for this booking have already been allocated. If you wish to change a participant, please do so from your <a href="' . $checkBookingUrl .'"><strong>Check My Booking</strong></a> section.'),
					'flash_fail'
				);

				return $this->redirect($this->referer());
			}

			$waivers = array();

			if (! empty($this->request->data['FirstName'])) {
				foreach ($this->request->data['FirstName'] as $key => $firstName) {
					$waivers[] = array(
						'sales_ref' => $salesRef,
						'booking_date' => $date,
						'first_name' => $firstName,
						'last_name' => $this->request->data['LastName'][$key],
						'email' => $this->request->data['Email'][$key]
					);
				}
			}

			if ($this->Participant->saveMany($waivers)) {
				$event = new CakeEvent('Model.Participant.afterCreate', $this, array(
	                'salesRef' => $salesRef
	            ));
	            $this->getEventManager()->dispatch($event);

				$this->Session->setFlash(
					__d('buzz_waivers', 'Emails with instructions on how to complete the waiver have been successfully sent to the participant(s) you entered.'),
					'flash_success'
				);

				return $this->redirect('/');
			}

		}
	}

	public function participants($id) {
		$this->loadModel('BuzzWaivers.Participant');
		$this->loadModel('BuzzWaivers.WaiversApi');
		$participant = $this->Participant->findById($id);

		if (empty($participant)) {
			throw new NotFoundException();
		}

		$participants = $this->Participant->find('all', array(
			'conditions' => array(
				'sales_ref' => $participant['Participant']['sales_ref']
			),
			'order' => array(
				'waiver_complete DESC',
				'first_name DESC',

			)
		));

		if (empty($participants)) {
			throw new NotFoundException();
		}

		$booking = $this->WaiversApi->getBooking(
			$participant['Participant']['sales_ref'],
			$participant['Participant']['booking_date']
		);

		if (empty($booking)) {
			$this->Session->setFlash(__d('buzz_waivers', 'A booking for the specified participant could not be found.'), 'flash_fail');
			throw new NotFoundException();
		}

		if ($this->request->is('post') || $this->request->is('put')) {

			// Save all the waivers again
			$this->loadModel('BuzzWaivers.Participant');

			// Check all the participants are those we found above.
			$validParticipants = Hash::extract($participants, '{n}.Participant.id');
			$postedParticipants = array_keys($this->request->data['Participant']);

			$failed = false;

			foreach ($this->request->data['Participant'] as $id => $participant) {
				if (in_array($id, $validParticipants)) {
					$this->Participant->clear();
					$this->Participant->id = $id;

					if (! $this->Participant->save($participant)) {
						$failed = true;
					}
				}
			}

			if ($failed) {
				$this->Session->setFlash(__d('buzz_waivers', 'There was a problem saving your waiver. Please check your details and try again.'), 'flash_fail');
				return $this->redirect($this->here);
			}

			// Check if an individual waiver is being signed - if so we'll
			// redirect to that one.
			if (! empty($this->request->data['sign_waiver'])) {
				$route = array(
					'admin' => false,
					'plugin' => 'buzz_waivers',
					'controller' => 'waivers',
					'action' => 'stage',
					'terms',
					$this->request->data['sign_waiver']
				);

				$route = Router::url($route);

				if ($this->kiosk) {
					$route = '/kiosk' . $route;
				}

				return $this->redirect($route);
			}

			$this->Session->setFlash(__d('buzz_waivers', 'Participant information has been successfully updated for this booking.'), 'flash_success');
			return $this->redirect($this->here);
		}

		// Split participants into those that have completed a waiver, and those
		// that have not.
		$completedParticipants = array();

		foreach ($participants as $key => $participant) {
			if ($participant['Participant']['waiver_complete'] == '1') {
				$completedParticipants[] = $participant;
				unset($participants[$key]);
			}
		}

		$this->set(compact('participants', 'completedParticipants', 'booking'));
	}
}
